/*#########################################################################
#   Amarok script for interfacing with icecast.org / dir.xiph.org.        #
#                                                                         #
#   Copyright                                                             #
#   For the original Librivox Scripts:                                    #
#   (C) 2008 Nikolaj Hald Nielsen <nhnFreespirit@gmail.com>               #
#   (C) 2008 Peter ZHOU <peterzhoulei@gmail.com>                          #
#   For the categories Array idea from the PlayOGG Scripts:               #
#   (C) 2009 Andrey Barrientos M.                                         #
#   For the filter and fetching logic from the Free Music Charts skript   #
#   (C) 2008-2009 Sven Krohlas <sven@getamarok.com>                       #
#   For the icecast.org / dir.xiph.org modifications:                     #
#   (C) 2009-2011 CyrusDreams <cyrusdreams ät freenet.de>                 #
#                                                                         #
#   This program is free software; you can redistribute it and/or modify  #
#   it under the terms of the GNU General Public License as published by  #
#   the Free Software Foundation; either version 2 of the License, or     #
#   (at your option) any later version.                                   #
#                                                                         #
#   This program is distributed in the hope that it will be useful,       #
#   but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
#   GNU General Public License for more details.                          #
#                                                                         #
#   You should have received a copy of the GNU General Public License     #
#   along with this program; if not, write to the                         #
#   Free Software Foundation, Inc.,                                       #
#   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         #
#########################################################################*/

Importer.loadQtBinding( "qt.core" );
Importer.loadQtBinding( "qt.xml" );
Importer.loadQtBinding( "qt.network" );
Importer.loadQtBinding( "qt.gui" ); //for QPixmap

xmlUrl    = new QUrl( "http://dir.xiph.org/yp.xml" );

QByteArray.prototype.toString = function() 
{
    ts = new QTextStream( this, QIODevice.ReadOnly );
    return ts.readAll();
}

function Station( server_name, listen_url, server_type,  bitrate, channels, samplerate, genre )
{
    this.server_name = server_name;
    this.listen_url = listen_url;
    this.server_type = server_type;
    this.bitrate = bitrate;
    this.channels = channels;
    this.samplerate = samplerate;
    this.genre = genre;
}

categories = new Object;

categories[""] = new Array (
    new Station(  )
);

categories["Ogg Vorbis stations"] = new Array ( );

categories["AAC+ stations"] = new Array ( );

categories["AAC stations"] = new Array ( );

categories["other stations"] = new Array ( );

categories["Icecast stations"] = new Array ( );


function Icecast()
{

    var currentDir = Amarok.Info.scriptPath() + "/";

    var file = new QFile( currentDir + "IcecastService.html" );
    file.open( QIODevice.OpenMode( QIODevice.ReadOnly, QIODevice.Text ) );

    while ( !file.atEnd() ) {
        html += file.readLine().toString();
    }

    html = html.replace( "_IMAGE_DIR_", currentDir );

    Amarok.debug ("Creating service...");
    //Amarok.debug ("html: " + html );

      if ( streamElements.length() == 0) { //we have to fetch the XML
        try
        {
          Amarok.debug( "Fetching stations XML..." );
          Amarok.Window.Statusbar.longMessage( "<b>Icecast.org service</b><br/><br/>Fetching stations.<br/>This might take some time, depending on the speed of your internet connection..." );
          a = new Downloader( xmlUrl, streamFetchResult );
        }
        catch( err )
        {
          Amarok.debug( err );
        }

        Amarok.debug ("Done fetching service!");
      }

    ScriptableServiceScript.call( this, "Icecast.org", 2, "Browse streams from the Icecast stream directory", html, true );

}

function onCustomize()
{
    Amarok.debug ("Customizing icecast service...");
    var currentDir = Amarok.Info.scriptPath() + "/";
    Amarok.debug ( "loading icon: " + currentDir + "IcecastIcon.png" );
    var iconPixmap = new QPixmap( currentDir + "IcecastIcon.png" );
    script.setIcon( iconPixmap );

    var emblemPixmap = new QPixmap( currentDir + "IcecastEmblem.png" );
    script.setEmblem( emblemPixmap );
    script.setScalableEmblem( currentDir + "IcecastScalableEmblem.svg" );
}

function streamFetchResult( reply ) {
// The function parses the yp.xml file and fills the various categories Arrays with Station functions. 
// Stations are part of more than one categories Array.
// The categories Arrays are built on-the-fly.

    try
    {
// reminder: Station ( server_name, listen_url, server_type,  bitrate, channels, samplerate, genre )
        if( streamElements.length() == 0) {
          doc.setContent( reply );
          streamElements = doc.elementsByTagName( "entry" );
          Amarok.debug ("got " + streamElements.length() + " streams!");
        }

        var snames = new Array( streamElements.length() );
        var lurls = new Array( streamElements.length() );
        var stypes = new Array( streamElements.length() );
        var brates = new Array( streamElements.length() );
        var chs = new Array( streamElements.length() );
        var srates = new Array( streamElements.length() );
        var gens = new Array( streamElements.length() );

        var i = 0;
        for ( ; i < streamElements.length(); i++ ) {

            elt = streamElements.at( i );

            elt2 = elt.firstChildElement( "server_name" );
            snames[i] = elt2.text();

            elt2 = elt.firstChildElement( "listen_url" );
            lurls[i] = elt2.text();

            elt2 = elt.firstChildElement( "server_type" );
            stypes[i] = elt2.text();

            elt2 = elt.firstChildElement( "bitrate" );
            brates[i] = elt2.text();

            elt2 = elt.firstChildElement( "channels" );
            chs[i] = elt2.text();

            elt2 = elt.firstChildElement( "samplerate" );
            srates[i] = elt2.text();

            elt2 = elt.firstChildElement( "genre" );
            gens[i] = elt2.text();

        }

        for ( i = 0; i < streamElements.length(); i++ ) {
            sname = snames[i];
            lurl = lurls[i];
            stype = stypes[i];
            brate = brates[i];
            ch = chs[i];
            srate = srates[i];
            gen = gens[i];
        Amarok.debug ("Built station No. " + i + " : " + sname + lurl + stype + brate + ch + srate + gen + " !");


/*            if ( stype == "application/ogg") {
                categories["Ogg Vorbis stations"].push(
                new Station (
                    sname,
                    lurl,
                    stype,
                    brate,
                    ch,
                    srate,
                    gen
                ));
            }
            else if ( stype == "audio/aacp") {
                categories["AAC+ stations"].push(
                new Station (
                    sname,
                    lurl,
                    stype,
                    brate,
                    ch,
                    srate,
                    gen
                ));
            }
            else if ( stype == "audio/aac") {
                categories["AAC stations"].push(
                new Station (
                    sname,
                    lurl,
                    stype,
                    brate,
                    ch,
                    srate,
                    gen
                ));
            }
            else {
                categories["other stations"].push(
                new Station (
                    sname,
                    lurl,
                    stype,
                    brate,
                    ch,
                    srate,
                    gen
                ));
            }
*/
/*station sorting is broken since Amarok 2.4, following "All stations" sorting is the back-up for now*/
                categories["Icecast stations"].push(
                new Station (
                    sname,
                    lurl,
                    stype,
                    brate,
                    ch,
                    srate,
                    gen
                ));
 
        }
    }
    catch( err )
    {
        Amarok.debug( err );
    }


}

function populateCategories () {

    var cover = Amarok.Info.scriptPath() + "/IcecastIcon.png";
    var xiphcover = Amarok.Info.scriptPath() + "/xiph.png";

    Amarok.debug ("Do categories !");
    try
    {
/*                item = Amarok.StreamItem;
                item.itemName = "Ogg Vorbis stations";
                item.infoHtml = html + "<center>List of " + "Ogg Vorbis stations" + "</center>";
                item.playableUrl = "";
                item.callbackData = "Ogg Vorbis stations";
                item.level = 1;
                item.coverUrl = xiphcover;
                script.insertItem( item );

                item = Amarok.StreamItem;
                item.itemName = "AAC+ stations";
                item.infoHtml = html + "<center>List of " + "AAC+ stations" + "</center>";
                item.playableUrl = "";
                item.callbackData = "AAC+ stations";
                item.level = 1;
                item.coverUrl = cover;
                script.insertItem( item );

                item = Amarok.StreamItem;
                item.itemName = "AAC stations";
                item.infoHtml = html + "<center>List of " + "AAC stations" + "</center>";
                item.playableUrl = "";
                item.callbackData = "AAC stations";
                item.level = 1;
                item.coverUrl = cover;
                script.insertItem( item );
                
                item = Amarok.StreamItem;
                item.itemName = "other stations";
                item.infoHtml = html + "<center>List of " + "other stations" + "</center>";
                item.playableUrl = "";
                item.callbackData = "other stations";
                item.level = 1;
                item.coverUrl = cover;
                script.insertItem( item );
*/

                item = Amarok.StreamItem;
                item.itemName = "Icecast stations";
                item.infoHtml = html + "<center>List of " + "other stations" + "</center>";
                item.playableUrl = "";
                item.callbackData = "Icecast stations";
                item.level = 1;
                item.coverUrl = cover;
                script.insertItem( item );



//        for (category in categories) {
//                item = Amarok.StreamItem;
//                item.itemName = category;
//                item.infoHtml = html + "<center>List of " + category + "</center>";
//                item.playableUrl = "";
//                item.callbackData = category;
//                item.level = 1;
//                item.coverUrl = cover;
//                script.insertItem( item );
//        }
    }
    catch( err )
    {
        Amarok.debug( err );
    }
    script.donePopulating(); 
}

function onPopulate( level, callbackData, filter ) {

  var cover = Amarok.Info.scriptPath() + "/IcecastIcon.png";

  filter        = filter.replace( "%20", " " );
  filter        = filter.trim();
  currentFilter = filter.toLowerCase();

    if ( level == 1 ) 
      populateCategories(); //since we have the XML, we directly filter the result
  // No script.donePopulating(); here, as the downloader returns
  // immediately, even before the parser is being run.
  // Instead call it at then end of fmcShowsXmlParser(...).

    else if ( level == 0 ) {
        try
        {
        Amarok.debug( " Populating streams with..." + callbackData);
            var stationArray = categories[callbackData];
            var i = 0;
            for ( ; i < stationArray.length; i++ )
                    {
                    item = Amarok.StreamItem;
                    item.level = 0;
                    item.callbackData = "";
                    item.itemName = stationArray[i].server_name;
                    item.playableUrl = stationArray[i].listen_url;
//                    item.infoHtml = html + "Station: " +...
                    item.infoHtml = "Station: " + stationArray[i].server_name + "<br />" + "Genre: " + stationArray[i].genre + "<br />" + "Stream Type: " + stationArray[i].server_type + "<br />" + "Quality: " + stationArray[i].bitrate + " kbps - " + stationArray[i].samplerate + " Hz" + "<br />" + "Channels: " + stationArray[i].channels;
//                    item.coverUrl = "";
                    if ( isFilterMatch ( item.itemName ) )
                      script.insertItem( item );
                    }
        Amarok.debug( " Done doing category..." + callbackData);
        }
        catch( err )
        {
            Amarok.debug( err );
        }
    }

    script.donePopulating();    
}

/*###########################################################################
#   Helper functions                                                        #
###########################################################################*/

/* checks weather a show contains a match for the current filter */
function containsFilterMatch( id ) {
  Amarok.debug( "FMC: containsFilterMatch" );

  if( currentFilter == "" ) // no filter -> matches everywhere
    return true;

  else {
    // podcast names are not in the xml
    var itemNameLowerCase = "podcast (mp3)";
    if( itemNameLowerCase.indexOf( currentFilter ) != -1 )
      return true;

    itemNameLowerCase = "podcast (ogg)";
    if( itemNameLowerCase.indexOf( currentFilter ) != -1 )
      return true;

    var tempElt = new QDomElement;
    tempElt = shows.at( id );

    // match the show title
    itemNameLowerCase = tempElt.firstChildElement( "title" ).text();
    itemNameLowerCase = itemNameLowerCase.toLowerCase();
    if( itemNameLowerCase.indexOf( currentFilter ) != -1 )
      return true;

    var i   = tempElt.firstChildElement( "songcount" ).text();
    tempElt = tempElt.firstChildElement( "songs" ); // ascend to songs
    tempElt = tempElt.firstChildElement( "song" );  // ascent to first song

    for( ; i != 0; i-- ) {
      itemNameLowerCase = tempElt.firstChildElement( "name" ).text();
      itemNameLowerCase = itemNameLowerCase.toLowerCase();

      if( itemNameLowerCase.indexOf( currentFilter ) != -1 )
        return true;

      tempElt = tempElt.nextSiblingElement( "song" );
    }
  }

  return false;
}


/* checks weather a song matches a filter */
function isFilterMatch( itemName ) {
  Amarok.debug( "FMC: isFilterMatch" );
  var itemNameLowerCase = itemName.toLowerCase();

  if( ( currentFilter != "" ) && ( itemNameLowerCase.indexOf( currentFilter ) == -1 ) )
    return false;

  return true;
}


String.prototype.trim = function() {
  a = this.replace(/^\s+/, '');
  return a.replace(/\s+$/, '');
}

/*###########################################################################
#   End Helper functions                                                    #
###########################################################################*/


http = new QHttp;
data = new QBuffer;
doc = new QDomDocument("doc");
elt = new QDomElement;
elt2 = new QDomElement;
streamElements = new QDomNodeList;
html = "";

script = new Icecast();
script.populate.connect( onPopulate );
script.customize.connect( onCustomize );
